@@ -19,7 +19,7 @@ pub fn main(config: &mut Config) -> CliResult {
                 return super::execute_external_subcommand(config, cmd, &[cmd, "--help"])
                     .map_err(|_| e.into());
             } else {
-                return Err(e)?;
+                return Err(e.into());
             }
         }
     };
@@ -36,6 +36,7 @@ Available unstable (nightly-only) flags:
     -Z config-profile   -- Read profiles from .cargo/config files
     -Z install-upgrade  -- `cargo install` will upgrade instead of failing
     -Z cache-messages   -- Cache compiler messages
+    -Z timings          -- Display concurrency information
 
 Run with 'cargo -Z [FLAG] [SUBCOMMAND]'"
         );
