@@ -1,9 +1,9 @@
-use core::compiler::{BuildConfig, CompileMode, Kind, TargetInfo};
-use core::{PackageSet, Resolve, Workspace};
-use ops;
+use crate::core::compiler::{BuildConfig, CompileMode, Kind, TargetInfo};
+use crate::core::{PackageSet, Resolve, Workspace};
+use crate::ops;
+use crate::util::CargoResult;
+use crate::util::Config;
 use std::collections::HashSet;
-use util::CargoResult;
-use util::Config;
 
 pub struct FetchOptions<'a> {
     pub config: &'a Config,
@@ -16,12 +16,13 @@ pub fn fetch<'a>(
     ws: &Workspace<'a>,
     options: &FetchOptions<'a>,
 ) -> CargoResult<(Resolve, PackageSet<'a>)> {
+    ws.emit_warnings()?;
     let (packages, resolve) = ops::resolve_ws(ws)?;
 
     let jobs = Some(1);
     let config = ws.config();
     let build_config = BuildConfig::new(config, jobs, &options.target, CompileMode::Build)?;
-    let rustc = config.rustc(Some(ws))?;
+    let rustc = config.load_global_rustc(Some(ws))?;
     let target_info =
         TargetInfo::new(config, &build_config.requested_target, &rustc, Kind::Target)?;
     {
