@@ -1,7 +1,3 @@
-use std::ffi::{OsStr, OsString};
-use std::fs;
-use std::path::PathBuf;
-
 use crate::core::compiler::{BuildConfig, MessageFormat};
 use crate::core::Workspace;
 use crate::ops::{CompileFilter, CompileOptions, NewOptions, Packages, VersionControl};
@@ -14,6 +10,10 @@ use crate::util::{
 };
 use crate::CargoResult;
 use clap::{self, SubCommand};
+use failure::bail;
+use std::ffi::{OsStr, OsString};
+use std::fs;
+use std::path::PathBuf;
 
 pub use crate::core::compiler::CompileMode;
 pub use crate::{CliError, CliResult, Config};
@@ -31,7 +31,8 @@ pub trait AppExt: Sized {
         exclude: &'static str,
     ) -> Self {
         self.arg_package_spec_simple(package)
-            ._arg(opt("all", all))
+            ._arg(opt("all", "Alias for --workspace (deprecated)"))
+            ._arg(opt("workspace", all))
             ._arg(multi_opt("exclude", "SPEC", exclude))
     }
 
@@ -99,9 +100,11 @@ pub trait AppExt: Sized {
     }
 
     fn arg_features(self) -> Self {
-        self._arg(
-            opt("features", "Space-separated list of features to activate").value_name("FEATURES"),
-        )
+        self._arg(multi_opt(
+            "features",
+            "FEATURES",
+            "Space-separated list of features to activate",
+        ))
         ._arg(opt("all-features", "Activate all available features"))
         ._arg(opt(
             "no-default-features",
@@ -132,13 +135,7 @@ pub trait AppExt: Sized {
     }
 
     fn arg_message_format(self) -> Self {
-        self._arg(
-            opt("message-format", "Error format")
-                .value_name("FMT")
-                .case_insensitive(true)
-                .possible_values(&["human", "json", "short"])
-                .default_value("human"),
-        )
+        self._arg(multi_opt("message-format", "FMT", "Error format"))
     }
 
     fn arg_build_plan(self) -> Self {
@@ -152,8 +149,7 @@ pub trait AppExt: Sized {
         self._arg(
             opt(
                 "vcs",
-                "\
-                 Initialize a new repository for the given version \
+                "Initialize a new repository for the given version \
                  control system (git, hg, pijul, or fossil) or do not \
                  initialize any version control at all (none), overriding \
                  a global configuration.",
@@ -295,28 +291,76 @@ pub trait ArgMatchesExt {
         workspace: Option<&Workspace<'a>>,
     ) -> CargoResult<CompileOptions<'a>> {
         let spec = Packages::from_flags(
-            self._is_present("all"),
+            // TODO Integrate into 'workspace'
+            self._is_present("workspace") || self._is_present("all"),
             self._values_of("exclude"),
             self._values_of("package"),
         )?;
 
-        let message_format = match self._value_of("message-format") {
-            None => MessageFormat::Human,
-            Some(f) => {
-                if f.eq_ignore_ascii_case("json") {
-                    MessageFormat::Json
-                } else if f.eq_ignore_ascii_case("human") {
-                    MessageFormat::Human
-                } else if f.eq_ignore_ascii_case("short") {
-                    MessageFormat::Short
-                } else {
-                    panic!("Impossible message format: {:?}", f)
+        let mut message_format = None;
+        let default_json = MessageFormat::Json {
+            short: false,
+            ansi: false,
+            render_diagnostics: false,
+        };
+        for fmt in self._values_of("message-format") {
+            for fmt in fmt.split(',') {
+                let fmt = fmt.to_ascii_lowercase();
+                match fmt.as_str() {
+                    "json" => {
+                        if message_format.is_some() {
+                            bail!("cannot specify two kinds of `message-format` arguments");
+                        }
+                        message_format = Some(default_json);
+                    }
+                    "human" => {
+                        if message_format.is_some() {
+                            bail!("cannot specify two kinds of `message-format` arguments");
+                        }
+                        message_format = Some(MessageFormat::Human);
+                    }
+                    "short" => {
+                        if message_format.is_some() {
+                            bail!("cannot specify two kinds of `message-format` arguments");
+                        }
+                        message_format = Some(MessageFormat::Short);
+                    }
+                    "json-render-diagnostics" => {
+                        if message_format.is_none() {
+                            message_format = Some(default_json);
+                        }
+                        match &mut message_format {
+                            Some(MessageFormat::Json {
+                                render_diagnostics, ..
+                            }) => *render_diagnostics = true,
+                            _ => bail!("cannot specify two kinds of `message-format` arguments"),
+                        }
+                    }
+                    "json-diagnostic-short" => {
+                        if message_format.is_none() {
+                            message_format = Some(default_json);
+                        }
+                        match &mut message_format {
+                            Some(MessageFormat::Json { short, .. }) => *short = true,
+                            _ => bail!("cannot specify two kinds of `message-format` arguments"),
+                        }
+                    }
+                    "json-diagnostic-rendered-ansi" => {
+                        if message_format.is_none() {
+                            message_format = Some(default_json);
+                        }
+                        match &mut message_format {
+                            Some(MessageFormat::Json { ansi, .. }) => *ansi = true,
+                            _ => bail!("cannot specify two kinds of `message-format` arguments"),
+                        }
+                    }
+                    s => bail!("invalid message format specifier: `{}`", s),
                 }
             }
-        };
+        }
 
         let mut build_config = BuildConfig::new(config, self.jobs()?, &self.target(), mode)?;
-        build_config.message_format = message_format;
+        build_config.message_format = message_format.unwrap_or(MessageFormat::Human);
         build_config.release = self._is_present("release");
         build_config.build_plan = self._is_present("build-plan");
         if build_config.build_plan {
@@ -516,8 +560,8 @@ pub enum CommandInfo {
 impl CommandInfo {
     pub fn name(&self) -> &str {
         match self {
-            CommandInfo::BuiltIn { name, .. } => &name,
-            CommandInfo::External { name, .. } => &name,
+            CommandInfo::BuiltIn { name, .. } => name,
+            CommandInfo::External { name, .. } => name,
         }
     }
 }
