@@ -4,7 +4,6 @@ use std::collections::hash_map::{Entry, HashMap};
 use std::env;
 use std::hash::{Hash, Hasher, SipHasher};
 use std::path::{Path, PathBuf};
-use std::process::Stdio;
 use std::sync::Mutex;
 
 use log::{debug, info, warn};
@@ -20,7 +19,7 @@ pub struct Rustc {
     pub path: PathBuf,
     /// An optional program that will be passed the path of the rust exe as its first argument, and
     /// rustc args following this.
-    pub wrapper: Option<PathBuf>,
+    pub wrapper: Option<ProcessBuilder>,
     /// Verbose version information (the output of `rustc -vV`)
     pub verbose_version: String,
     /// The host triple (arch-platform-OS), this comes from verbose_version.
@@ -53,13 +52,18 @@ impl Rustc {
                 .lines()
                 .find(|l| l.starts_with("host: "))
                 .map(|l| &l[6..])
-                .ok_or_else(|| internal("rustc -v didn't have a line for `host:`"))?;
+                .ok_or_else(|| {
+                    failure::format_err!(
+                        "`rustc -vV` didn't have a line for `host:`, got:\n{}",
+                        verbose_version
+                    )
+                })?;
             triple.to_string()
         };
 
         Ok(Rustc {
             path,
-            wrapper,
+            wrapper: wrapper.map(util::process),
             verbose_version,
             host,
             cache: Mutex::new(cache),
@@ -67,17 +71,22 @@ impl Rustc {
     }
 
     /// Gets a process builder set up to use the found rustc version, with a wrapper if `Some`.
-    pub fn process(&self) -> ProcessBuilder {
+    pub fn process_with(&self, path: impl AsRef<Path>) -> ProcessBuilder {
         match self.wrapper {
-            Some(ref wrapper) if !wrapper.as_os_str().is_empty() => {
-                let mut cmd = util::process(wrapper);
-                cmd.arg(&self.path);
+            Some(ref wrapper) if !wrapper.get_program().is_empty() => {
+                let mut cmd = wrapper.clone();
+                cmd.arg(path.as_ref());
                 cmd
             }
-            _ => self.process_no_wrapper(),
+            _ => util::process(path.as_ref()),
         }
     }
 
+    /// Gets a process builder set up to use the found rustc version, with a wrapper if `Some`.
+    pub fn process(&self) -> ProcessBuilder {
+        self.process_with(&self.path)
+    }
+
     pub fn process_no_wrapper(&self) -> ProcessBuilder {
         util::process(&self.path)
     }
@@ -86,16 +95,16 @@ impl Rustc {
         self.cache.lock().unwrap().cached_output(cmd)
     }
 
-    pub fn cached_success(&self, cmd: &ProcessBuilder) -> CargoResult<bool> {
-        self.cache.lock().unwrap().cached_success(cmd)
+    pub fn set_wrapper(&mut self, wrapper: ProcessBuilder) {
+        self.wrapper = Some(wrapper);
     }
 }
 
-/// It is a well known that `rustc` is not the fastest compiler in the world.
-/// What is less known is that even `rustc --version --verbose` takes about a
-/// hundred milliseconds! Because we need compiler version info even for no-op
-/// builds, we cache it here, based on compiler's mtime and rustup's current
-/// toolchain.
+/// It is a well known fact that `rustc` is not the fastest compiler in the
+/// world.  What is less known is that even `rustc --version --verbose` takes
+/// about a hundred milliseconds! Because we need compiler version info even
+/// for no-op builds, we cache it here, based on compiler's mtime and rustup's
+/// current toolchain.
 ///
 /// https://github.com/rust-lang/cargo/issues/5315
 /// https://github.com/rust-lang/rust/issues/49761
@@ -126,16 +135,16 @@ impl Cache {
                 let data = match read(&cache_location) {
                     Ok(data) => {
                         if data.rustc_fingerprint == rustc_fingerprint {
-                            info!("reusing existing rustc info cache");
+                            debug!("reusing existing rustc info cache");
                             dirty = false;
                             data
                         } else {
-                            info!("different compiler, creating new rustc info cache");
+                            debug!("different compiler, creating new rustc info cache");
                             empty
                         }
                     }
                     Err(e) => {
-                        info!("failed to read rustc info cache: {}", e);
+                        debug!("failed to read rustc info cache: {}", e);
                         empty
                     }
                 };
@@ -154,7 +163,7 @@ impl Cache {
                 if let Err(e) = fingerprint {
                     warn!("failed to calculate rustc fingerprint: {}", e);
                 }
-                info!("rustc info cache disabled");
+                debug!("rustc info cache disabled");
                 Cache {
                     cache_location: None,
                     dirty: false,
@@ -168,11 +177,12 @@ impl Cache {
         let key = process_fingerprint(cmd);
         match self.data.outputs.entry(key) {
             Entry::Occupied(entry) => {
-                info!("rustc info cache hit");
+                debug!("rustc info cache hit");
                 Ok(entry.get().clone())
             }
             Entry::Vacant(entry) => {
-                info!("rustc info cache miss");
+                debug!("rustc info cache miss");
+                debug!("running {}", cmd);
                 let output = cmd.exec_with_output()?;
                 let stdout = String::from_utf8(output.stdout)
                     .map_err(|_| internal("rustc didn't return utf8 output"))?;
@@ -185,28 +195,6 @@ impl Cache {
             }
         }
     }
-
-    fn cached_success(&mut self, cmd: &ProcessBuilder) -> CargoResult<bool> {
-        let key = process_fingerprint(cmd);
-        match self.data.successes.entry(key) {
-            Entry::Occupied(entry) => {
-                info!("rustc info cache hit");
-                Ok(*entry.get())
-            }
-            Entry::Vacant(entry) => {
-                info!("rustc info cache miss");
-                let success = cmd
-                    .build_command()
-                    .stdout(Stdio::null())
-                    .stderr(Stdio::null())
-                    .status()?
-                    .success();
-                entry.insert(success);
-                self.dirty = true;
-                Ok(success)
-            }
-        }
-    }
 }
 
 impl Drop for Cache {
