@@ -1,11 +1,11 @@
-use crate::support::publish::validate_alt_upload;
-use crate::support::registry::{self, Package};
-use crate::support::{basic_manifest, git, paths, project};
-use cargo::util::ToUrl;
+use cargo::util::IntoUrl;
+use cargo_test_support::publish::validate_alt_upload;
+use cargo_test_support::registry::{self, Package};
+use cargo_test_support::{basic_manifest, git, paths, project};
 use std::fs::{self, File};
 use std::io::Write;
 
-#[test]
+#[cargo_test]
 fn depend_on_alt_registry() {
     let p = project()
         .file(
@@ -54,7 +54,7 @@ fn depend_on_alt_registry() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn depend_on_alt_registry_depends_on_same_registry_no_index() {
     let p = project()
         .file(
@@ -96,7 +96,7 @@ fn depend_on_alt_registry_depends_on_same_registry_no_index() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn depend_on_alt_registry_depends_on_same_registry() {
     let p = project()
         .file(
@@ -138,7 +138,7 @@ fn depend_on_alt_registry_depends_on_same_registry() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn depend_on_alt_registry_depends_on_crates_io() {
     let p = project()
         .file(
@@ -182,7 +182,7 @@ fn depend_on_alt_registry_depends_on_crates_io() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn registry_and_path_dep_works() {
     registry::init();
 
@@ -216,7 +216,7 @@ fn registry_and_path_dep_works() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn registry_incompatible_with_git() {
     registry::init();
 
@@ -246,10 +246,10 @@ fn registry_incompatible_with_git() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn cannot_publish_to_crates_io_with_registry_dependency() {
     let fakeio_path = paths::root().join("fake.io");
-    let fakeio_url = fakeio_path.to_url().unwrap();
+    let fakeio_url = fakeio_path.into_url().unwrap();
     let p = project()
         .file(
             "Cargo.toml",
@@ -304,7 +304,7 @@ fn cannot_publish_to_crates_io_with_registry_dependency() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn publish_with_registry_dependency() {
     let p = project()
         .file(
@@ -361,11 +361,11 @@ fn publish_with_registry_dependency() {
             "vers": "0.0.1"
         }"#,
         "foo-0.0.1.crate",
-        &["Cargo.toml", "Cargo.toml.orig", "src/main.rs"],
+        &["Cargo.lock", "Cargo.toml", "Cargo.toml.orig", "src/main.rs"],
     );
 }
 
-#[test]
+#[cargo_test]
 fn alt_registry_and_crates_io_deps() {
     let p = project()
         .file(
@@ -410,7 +410,7 @@ fn alt_registry_and_crates_io_deps() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn block_publish_due_to_no_token() {
     let p = project().file("src/main.rs", "fn main() {}").build();
 
@@ -426,7 +426,7 @@ fn block_publish_due_to_no_token() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn publish_to_alt_registry() {
     let p = project().file("src/main.rs", "fn main() {}").build();
 
@@ -460,11 +460,11 @@ fn publish_to_alt_registry() {
             "vers": "0.0.1"
         }"#,
         "foo-0.0.1.crate",
-        &["Cargo.toml", "Cargo.toml.orig", "src/main.rs"],
+        &["Cargo.lock", "Cargo.toml", "Cargo.toml.orig", "src/main.rs"],
     );
 }
 
-#[test]
+#[cargo_test]
 fn publish_with_crates_io_dep() {
     let p = project()
         .file(
@@ -523,11 +523,11 @@ fn publish_with_crates_io_dep() {
             "vers": "0.0.1"
         }"#,
         "foo-0.0.1.crate",
-        &["Cargo.toml", "Cargo.toml.orig", "src/main.rs"],
+        &["Cargo.lock", "Cargo.toml", "Cargo.toml.orig", "src/main.rs"],
     );
 }
 
-#[test]
+#[cargo_test]
 fn passwords_in_registry_index_url_forbidden() {
     registry::init();
 
@@ -551,7 +551,7 @@ fn passwords_in_registry_index_url_forbidden() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn passwords_in_registries_index_url_forbidden() {
     registry::init();
 
@@ -575,7 +575,7 @@ fn passwords_in_registries_index_url_forbidden() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn patch_alt_reg() {
     Package::new("bar", "0.1.0").publish();
     let p = project()
@@ -616,7 +616,7 @@ fn patch_alt_reg() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bad_registry_name() {
     let p = project()
         .file(
@@ -664,7 +664,7 @@ Caused by:
     }
 }
 
-#[test]
+#[cargo_test]
 fn no_api() {
     Package::new("bar", "0.0.1").alternative(true).publish();
     // Configure without `api`.
@@ -746,7 +746,7 @@ fn no_api() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn alt_reg_metadata() {
     // Check for "registry" entries in `cargo metadata` with alternative registries.
     let p = project()
@@ -824,6 +824,7 @@ fn alt_reg_metadata() {
                         "features": {},
                         "manifest_path": "[..]/foo/Cargo.toml",
                         "metadata": null,
+                        "publish": null,
                         "authors": [],
                         "categories": [],
                         "keywords": [],
@@ -863,6 +864,7 @@ fn alt_reg_metadata() {
                         "features": {},
                         "manifest_path": "[..]/altdep2-0.0.1/Cargo.toml",
                         "metadata": null,
+                        "publish": null,
                         "authors": [],
                         "categories": [],
                         "keywords": [],
@@ -897,6 +899,7 @@ fn alt_reg_metadata() {
                         "features": {},
                         "manifest_path": "[..]/altdep-0.0.1/Cargo.toml",
                         "metadata": null,
+                        "publish": null,
                         "authors": [],
                         "categories": [],
                         "keywords": [],
@@ -943,6 +946,7 @@ fn alt_reg_metadata() {
                         "features": {},
                         "manifest_path": "[..]/foo/Cargo.toml",
                         "metadata": null,
+                        "publish": null,
                         "authors": [],
                         "categories": [],
                         "keywords": [],
@@ -977,6 +981,7 @@ fn alt_reg_metadata() {
                         "features": {},
                         "manifest_path": "[..]/iodep-0.0.1/Cargo.toml",
                         "metadata": null,
+                        "publish": null,
                         "authors": [],
                         "categories": [],
                         "keywords": [],
@@ -998,6 +1003,7 @@ fn alt_reg_metadata() {
                         "features": {},
                         "manifest_path": "[..]/bar-0.0.1/Cargo.toml",
                         "metadata": null,
+                        "publish": null,
                         "authors": [],
                         "categories": [],
                         "keywords": [],
@@ -1019,7 +1025,7 @@ fn alt_reg_metadata() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn unknown_registry() {
     // A known registry refers to an unknown registry.
     // foo -> bar(crates.io) -> baz(alt)
@@ -1076,6 +1082,7 @@ fn unknown_registry() {
                   "features": {},
                   "manifest_path": "[..]",
                   "metadata": null,
+                  "publish": null,
                   "authors": [],
                   "categories": [],
                   "keywords": [],
@@ -1110,6 +1117,7 @@ fn unknown_registry() {
                   "features": {},
                   "manifest_path": "[..]/foo/Cargo.toml",
                   "metadata": null,
+                  "publish": null,
                   "authors": [],
                   "categories": [],
                   "keywords": [],
@@ -1144,6 +1152,7 @@ fn unknown_registry() {
                   "features": {},
                   "manifest_path": "[..]",
                   "metadata": null,
+                  "publish": null,
                   "authors": [],
                   "categories": [],
                   "keywords": [],
@@ -1166,7 +1175,7 @@ fn unknown_registry() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn registries_index_relative_url() {
     let config = paths::root().join(".cargo/config");
     fs::create_dir_all(config.parent().unwrap()).unwrap();
@@ -1216,7 +1225,7 @@ fn registries_index_relative_url() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn registry_index_relative_url() {
     let config = paths::root().join(".cargo/config");
     fs::create_dir_all(config.parent().unwrap()).unwrap();
@@ -1268,7 +1277,7 @@ warning: custom registry support via the `registry.index` configuration is being
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn registries_index_relative_path_not_allowed() {
     let config = paths::root().join(".cargo/config");
     fs::create_dir_all(config.parent().unwrap()).unwrap();
