@@ -1,6 +1,6 @@
-use crate::support::is_nightly;
-use crate::support::paths::CargoPathExt;
-use crate::support::{basic_bin_manifest, basic_lib_manifest, basic_manifest, project};
+use cargo_test_support::is_nightly;
+use cargo_test_support::paths::CargoPathExt;
+use cargo_test_support::{basic_bin_manifest, basic_lib_manifest, basic_manifest, project};
 
 #[cargo_test]
 fn cargo_bench_simple() {
@@ -58,7 +58,7 @@ fn bench_bench_implicit() {
         .file(
             "src/main.rs",
             r#"
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             #[cfg(test)]
             extern crate test;
             #[bench] fn run1(_ben: &mut test::Bencher) { }
@@ -364,7 +364,7 @@ fn bench_with_lib_dep() {
         .file(
             "src/lib.rs",
             r#"
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             #[cfg(test)]
             extern crate test;
             ///
@@ -432,7 +432,7 @@ fn bench_with_deep_lib_dep() {
         .file(
             "src/lib.rs",
             "
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             #[cfg(test)]
             extern crate foo;
             #[cfg(test)]
@@ -448,7 +448,7 @@ fn bench_with_deep_lib_dep() {
         .file(
             "src/lib.rs",
             "
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             #[cfg(test)]
             extern crate test;
 
@@ -495,7 +495,7 @@ fn external_bench_explicit() {
         .file(
             "src/lib.rs",
             r#"
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             #[cfg(test)]
             extern crate test;
             pub fn get_hello() -> &'static str { "Hello" }
@@ -541,7 +541,7 @@ fn external_bench_implicit() {
         .file(
             "src/lib.rs",
             r#"
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             #[cfg(test)]
             extern crate test;
 
@@ -760,7 +760,7 @@ fn lib_bin_same_name() {
         .file(
             "src/lib.rs",
             "
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             #[cfg(test)]
             extern crate test;
             #[bench] fn lib_bench(_b: &mut test::Bencher) {}
@@ -769,7 +769,7 @@ fn lib_bin_same_name() {
         .file(
             "src/main.rs",
             "
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             #[allow(unused_extern_crates)]
             extern crate foo;
             #[cfg(test)]
@@ -804,7 +804,7 @@ fn lib_with_standard_name() {
         .file(
             "src/lib.rs",
             "
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             #[cfg(test)]
             extern crate test;
 
@@ -919,7 +919,7 @@ fn bench_dylib() {
         .file(
             "src/lib.rs",
             r#"
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             extern crate bar as the_bar;
             #[cfg(test)]
             extern crate test;
@@ -1061,7 +1061,7 @@ fn bench_with_examples() {
         .file(
             "src/lib.rs",
             r#"
-            #![cfg_attr(test, feature(test))]
+            #![feature(test)]
             #[cfg(test)]
             extern crate test;
             #[cfg(test)]
@@ -1382,7 +1382,7 @@ fn bench_all_workspace() {
         )
         .build();
 
-    p.cargo("bench --all")
+    p.cargo("bench --workspace")
         .with_stderr_contains("[RUNNING] target/release/deps/bar-[..][EXE]")
         .with_stdout_contains("test bench_bar ... bench: [..]")
         .with_stderr_contains("[RUNNING] target/release/deps/foo-[..][EXE]")
@@ -1430,7 +1430,7 @@ fn bench_all_exclude() {
         )
         .build();
 
-    p.cargo("bench --all --exclude baz")
+    p.cargo("bench --workspace --exclude baz")
         .with_stdout_contains(
             "\
 running 1 test
@@ -1484,7 +1484,7 @@ fn bench_all_virtual_manifest() {
         .build();
 
     // The order in which bar and baz are built is not guaranteed
-    p.cargo("bench --all")
+    p.cargo("bench --workspace")
         .with_stderr_contains("[RUNNING] target/release/deps/baz-[..][EXE]")
         .with_stdout_contains("test bench_baz ... bench: [..]")
         .with_stderr_contains("[RUNNING] target/release/deps/bar-[..][EXE]")
