@@ -1,7 +1,7 @@
 use std::env;
 
-use crate::support::registry::Package;
-use crate::support::{basic_bin_manifest, basic_manifest, git, main_file, project};
+use cargo_test_support::registry::Package;
+use cargo_test_support::{basic_bin_manifest, basic_manifest, git, main_file, project};
 
 #[cargo_test]
 fn cargo_clean_simple() {
@@ -212,8 +212,7 @@ fn clean_git() {
         project
             .file("Cargo.toml", &basic_manifest("dep", "0.5.0"))
             .file("src/lib.rs", "")
-    })
-    .unwrap();
+    });
 
     let p = project()
         .file(
