@@ -1,7 +1,7 @@
-use crate::support::{basic_bin_manifest, basic_manifest, cross_compile, project};
-use crate::support::{is_nightly, rustc_host};
+use cargo_test_support::{basic_bin_manifest, basic_manifest, cross_compile, project};
+use cargo_test_support::{is_nightly, rustc_host};
 
-#[test]
+#[cargo_test]
 fn simple_cross() {
     if cross_compile::disabled() {
         return;
@@ -50,7 +50,7 @@ fn simple_cross() {
     p.process(&p.target_bin(&target, "foo")).run();
 }
 
-#[test]
+#[cargo_test]
 fn simple_cross_config() {
     if cross_compile::disabled() {
         return;
@@ -109,7 +109,7 @@ fn simple_cross_config() {
     p.process(&p.target_bin(&target, "foo")).run();
 }
 
-#[test]
+#[cargo_test]
 fn simple_deps() {
     if cross_compile::disabled() {
         return;
@@ -143,12 +143,13 @@ fn simple_deps() {
     p.process(&p.target_bin(&target, "foo")).run();
 }
 
-#[test]
+#[cargo_test]
 fn plugin_deps() {
     if cross_compile::disabled() {
         return;
     }
     if !is_nightly() {
+        // plugins are unstable
         return;
     }
 
@@ -200,22 +201,21 @@ fn plugin_deps() {
             r#"
             #![feature(plugin_registrar, rustc_private)]
 
-            extern crate rustc_plugin;
+            extern crate rustc_driver;
             extern crate syntax;
 
-            use rustc_plugin::Registry;
-            use syntax::tokenstream::TokenTree;
+            use rustc_driver::plugin::Registry;
+            use syntax::tokenstream::TokenStream;
             use syntax::source_map::Span;
             use syntax::ast::*;
             use syntax::ext::base::{ExtCtxt, MacEager, MacResult};
-            use syntax::ext::build::AstBuilder;
 
             #[plugin_registrar]
             pub fn foo(reg: &mut Registry) {
                 reg.register_macro("bar", expand_bar);
             }
 
-            fn expand_bar(cx: &mut ExtCtxt, sp: Span, tts: &[TokenTree])
+            fn expand_bar(cx: &mut ExtCtxt, sp: Span, tts: TokenStream)
                           -> Box<MacResult + 'static> {
                 MacEager::expr(cx.expr_lit(sp, LitKind::Int(1, LitIntType::Unsuffixed)))
             }
@@ -235,12 +235,13 @@ fn plugin_deps() {
     foo.process(&foo.target_bin(&target, "foo")).run();
 }
 
-#[test]
+#[cargo_test]
 fn plugin_to_the_max() {
     if cross_compile::disabled() {
         return;
     }
     if !is_nightly() {
+        // plugins are unstable
         return;
     }
 
@@ -295,16 +296,15 @@ fn plugin_to_the_max() {
             r#"
             #![feature(plugin_registrar, rustc_private)]
 
-            extern crate rustc_plugin;
+            extern crate rustc_driver;
             extern crate syntax;
             extern crate baz;
 
-            use rustc_plugin::Registry;
-            use syntax::tokenstream::TokenTree;
+            use rustc_driver::plugin::Registry;
+            use syntax::tokenstream::TokenStream;
             use syntax::source_map::Span;
             use syntax::ast::*;
             use syntax::ext::base::{ExtCtxt, MacEager, MacResult};
-            use syntax::ext::build::AstBuilder;
             use syntax::ptr::P;
 
             #[plugin_registrar]
@@ -312,7 +312,7 @@ fn plugin_to_the_max() {
                 reg.register_macro("bar", expand_bar);
             }
 
-            fn expand_bar(cx: &mut ExtCtxt, sp: Span, tts: &[TokenTree])
+            fn expand_bar(cx: &mut ExtCtxt, sp: Span, tts: TokenStream)
                           -> Box<MacResult + 'static> {
                 let bar = Ident::from_str("baz");
                 let path = cx.path(sp, vec![bar.clone(), bar]);
@@ -336,7 +336,7 @@ fn plugin_to_the_max() {
     foo.process(&foo.target_bin(&target, "foo")).run();
 }
 
-#[test]
+#[cargo_test]
 fn linker_and_ar() {
     if cross_compile::disabled() {
         return;
@@ -377,7 +377,7 @@ fn linker_and_ar() {
             "\
 [COMPILING] foo v0.5.0 ([CWD])
 [RUNNING] `rustc --crate-name foo src/foo.rs --color never --crate-type bin \
-    --emit=dep-info,link -C debuginfo=2 \
+    --emit=[..]link -C debuginfo=2 \
     -C metadata=[..] \
     --out-dir [CWD]/target/{target}/debug/deps \
     --target {target} \
@@ -390,12 +390,13 @@ fn linker_and_ar() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn plugin_with_extra_dylib_dep() {
     if cross_compile::disabled() {
         return;
     }
     if !is_nightly() {
+        // plugins are unstable
         return;
     }
 
@@ -445,10 +446,10 @@ fn plugin_with_extra_dylib_dep() {
             r#"
             #![feature(plugin_registrar, rustc_private)]
 
-            extern crate rustc_plugin;
             extern crate baz;
+            extern crate rustc_driver;
 
-            use rustc_plugin::Registry;
+            use rustc_driver::plugin::Registry;
 
             #[plugin_registrar]
             pub fn foo(reg: &mut Registry) {
@@ -479,7 +480,7 @@ fn plugin_with_extra_dylib_dep() {
     foo.cargo("build --target").arg(&target).run();
 }
 
-#[test]
+#[cargo_test]
 fn cross_tests() {
     if cross_compile::disabled() {
         return;
@@ -542,7 +543,7 @@ fn cross_tests() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn no_cross_doctests() {
     if cross_compile::disabled() {
         return;
@@ -600,7 +601,7 @@ fn no_cross_doctests() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn simple_cargo_run() {
     if cross_compile::disabled() {
         return;
@@ -625,7 +626,7 @@ fn simple_cargo_run() {
     p.cargo("run --target").arg(&target).run();
 }
 
-#[test]
+#[cargo_test]
 fn cross_with_a_build_script() {
     if cross_compile::disabled() {
         return;
@@ -687,7 +688,7 @@ fn cross_with_a_build_script() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_script_needed_for_host_and_target() {
     if cross_compile::disabled() {
         return;
@@ -790,15 +791,14 @@ fn build_script_needed_for_host_and_target() {
             host = host
         ))
         .with_stderr_contains(&format!(
-            "\
-             [RUNNING] `rustc [..] src/main.rs [..] --target {target} [..] \
+            "[RUNNING] `rustc [..] src/main.rs [..] --target {target} [..] \
              -L /path/to/{target}`",
             target = target
         ))
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_deps_for_the_right_arch() {
     if cross_compile::disabled() {
         return;
@@ -841,7 +841,7 @@ fn build_deps_for_the_right_arch() {
     p.cargo("build -v --target").arg(&target).run();
 }
 
-#[test]
+#[cargo_test]
 fn build_script_only_host() {
     if cross_compile::disabled() {
         return;
@@ -892,7 +892,7 @@ fn build_script_only_host() {
     p.cargo("build -v --target").arg(&target).run();
 }
 
-#[test]
+#[cargo_test]
 fn plugin_build_script_right_arch() {
     if cross_compile::disabled() {
         return;
@@ -930,7 +930,7 @@ fn plugin_build_script_right_arch() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_script_with_platform_specific_dependencies() {
     if cross_compile::disabled() {
         return;
@@ -1003,7 +1003,7 @@ fn build_script_with_platform_specific_dependencies() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn platform_specific_dependencies_do_not_leak() {
     if cross_compile::disabled() {
         return;
@@ -1057,7 +1057,7 @@ fn platform_specific_dependencies_do_not_leak() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn platform_specific_variables_reflected_in_build_scripts() {
     if cross_compile::disabled() {
         return;
@@ -1143,7 +1143,7 @@ fn platform_specific_variables_reflected_in_build_scripts() {
     p.cargo("build -v --target").arg(&target).run();
 }
 
-#[test]
+#[cargo_test]
 fn cross_test_dylib() {
     if cross_compile::disabled() {
         return;
