@@ -1,8 +1,8 @@
-use crate::support::registry::Package;
-use crate::support::rustc_host;
-use crate::support::{basic_manifest, cross_compile, project};
+use cargo_test_support::registry::Package;
+use cargo_test_support::rustc_host;
+use cargo_test_support::{basic_manifest, cross_compile, project};
 
-#[test]
+#[cargo_test]
 fn no_deps() {
     let p = project()
         .file("src/main.rs", "mod a; fn main() {}")
@@ -12,7 +12,7 @@ fn no_deps() {
     p.cargo("fetch").with_stdout("").run();
 }
 
-#[test]
+#[cargo_test]
 fn fetch_all_platform_dependencies_when_no_target_is_given() {
     if cross_compile::disabled() {
         return;
@@ -59,7 +59,7 @@ fn fetch_all_platform_dependencies_when_no_target_is_given() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn fetch_platform_specific_dependencies() {
     if cross_compile::disabled() {
         return;
@@ -112,3 +112,22 @@ fn fetch_platform_specific_dependencies() {
         .with_stderr_does_not_contain("[DOWNLOADED] d1 v1.2.3 [..]")
         .run();
 }
+
+#[cargo_test]
+fn fetch_warning() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "1.0.0"
+            misspelled = "wut"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+    p.cargo("fetch")
+        .with_stderr("[WARNING] unused manifest key: package.misspelled")
+        .run();
+}
