@@ -1,10 +1,12 @@
 use std::fs::{self, File};
 use std::io::Write;
 
-use crate::support::rustc_host;
-use crate::support::{basic_lib_manifest, basic_manifest, paths, project, project_in_home};
+use cargo_test_support::registry::Package;
+use cargo_test_support::{
+    basic_lib_manifest, basic_manifest, paths, project, project_in_home, rustc_host,
+};
 
-#[test]
+#[cargo_test]
 fn env_rustflags_normal_source() {
     let p = project()
         .file("src/lib.rs", "")
@@ -48,7 +50,7 @@ fn env_rustflags_normal_source() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_build_script() {
     // RUSTFLAGS should be passed to rustc for build scripts
     // when --target is not specified.
@@ -77,7 +79,7 @@ fn env_rustflags_build_script() {
     p.cargo("build").env("RUSTFLAGS", "--cfg foo").run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_build_script_dep() {
     // RUSTFLAGS should be passed to rustc for build scripts
     // when --target is not specified.
@@ -114,7 +116,7 @@ fn env_rustflags_build_script_dep() {
     foo.cargo("build").env("RUSTFLAGS", "--cfg foo").run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_plugin() {
     // RUSTFLAGS should be passed to rustc for plugins
     // when --target is not specified.
@@ -145,7 +147,7 @@ fn env_rustflags_plugin() {
     p.cargo("build").env("RUSTFLAGS", "--cfg foo").run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_plugin_dep() {
     // RUSTFLAGS should be passed to rustc for plugins
     // when --target is not specified.
@@ -184,7 +186,7 @@ fn env_rustflags_plugin_dep() {
     foo.cargo("build").env("RUSTFLAGS", "--cfg foo").run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_normal_source_with_target() {
     let p = project()
         .file("src/lib.rs", "")
@@ -235,7 +237,7 @@ fn env_rustflags_normal_source_with_target() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_build_script_with_target() {
     // RUSTFLAGS should not be passed to rustc for build scripts
     // when --target is specified.
@@ -268,7 +270,7 @@ fn env_rustflags_build_script_with_target() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_build_script_dep_with_target() {
     // RUSTFLAGS should not be passed to rustc for build scripts
     // when --target is specified.
@@ -309,7 +311,7 @@ fn env_rustflags_build_script_dep_with_target() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_plugin_with_target() {
     // RUSTFLAGS should not be passed to rustc for plugins
     // when --target is specified.
@@ -344,7 +346,7 @@ fn env_rustflags_plugin_with_target() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_plugin_dep_with_target() {
     // RUSTFLAGS should not be passed to rustc for plugins
     // when --target is specified.
@@ -387,7 +389,7 @@ fn env_rustflags_plugin_dep_with_target() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_recompile() {
     let p = project().file("src/lib.rs", "").build();
 
@@ -400,7 +402,7 @@ fn env_rustflags_recompile() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_recompile2() {
     let p = project().file("src/lib.rs", "").build();
 
@@ -413,7 +415,7 @@ fn env_rustflags_recompile2() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_no_recompile() {
     let p = project().file("src/lib.rs", "").build();
 
@@ -424,7 +426,7 @@ fn env_rustflags_no_recompile() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_normal_source() {
     let p = project()
         .file("src/lib.rs", "")
@@ -469,7 +471,7 @@ fn build_rustflags_normal_source() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_build_script() {
     // RUSTFLAGS should be passed to rustc for build scripts
     // when --target is not specified.
@@ -505,7 +507,7 @@ fn build_rustflags_build_script() {
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_build_script_dep() {
     // RUSTFLAGS should be passed to rustc for build scripts
     // when --target is not specified.
@@ -549,7 +551,7 @@ fn build_rustflags_build_script_dep() {
     foo.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_plugin() {
     // RUSTFLAGS should be passed to rustc for plugins
     // when --target is not specified.
@@ -587,7 +589,7 @@ fn build_rustflags_plugin() {
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_plugin_dep() {
     // RUSTFLAGS should be passed to rustc for plugins
     // when --target is not specified.
@@ -633,7 +635,7 @@ fn build_rustflags_plugin_dep() {
     foo.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_normal_source_with_target() {
     let p = project()
         .file("src/lib.rs", "")
@@ -686,7 +688,7 @@ fn build_rustflags_normal_source_with_target() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_build_script_with_target() {
     // RUSTFLAGS should not be passed to rustc for build scripts
     // when --target is specified.
@@ -723,7 +725,7 @@ fn build_rustflags_build_script_with_target() {
     p.cargo("build --target").arg(host).run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_build_script_dep_with_target() {
     // RUSTFLAGS should not be passed to rustc for build scripts
     // when --target is specified.
@@ -768,7 +770,7 @@ fn build_rustflags_build_script_dep_with_target() {
     foo.cargo("build --target").arg(host).run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_plugin_with_target() {
     // RUSTFLAGS should not be passed to rustc for plugins
     // when --target is specified.
@@ -807,7 +809,7 @@ fn build_rustflags_plugin_with_target() {
     p.cargo("build --target").arg(host).run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_plugin_dep_with_target() {
     // RUSTFLAGS should not be passed to rustc for plugins
     // when --target is specified.
@@ -854,7 +856,7 @@ fn build_rustflags_plugin_dep_with_target() {
     foo.cargo("build --target").arg(host).run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_recompile() {
     let p = project().file("src/lib.rs", "").build();
 
@@ -876,7 +878,7 @@ fn build_rustflags_recompile() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_recompile2() {
     let p = project().file("src/lib.rs", "").build();
 
@@ -898,7 +900,7 @@ fn build_rustflags_recompile2() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_no_recompile() {
     let p = project()
         .file("src/lib.rs", "")
@@ -918,7 +920,7 @@ fn build_rustflags_no_recompile() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_rustflags_with_home_config() {
     // We need a config file inside the home directory
     let home = paths::home();
@@ -941,7 +943,7 @@ fn build_rustflags_with_home_config() {
     p.cargo("build -v").run();
 }
 
-#[test]
+#[cargo_test]
 fn target_rustflags_normal_source() {
     let p = project()
         .file("src/lib.rs", "")
@@ -990,7 +992,7 @@ fn target_rustflags_normal_source() {
 }
 
 // target.{}.rustflags takes precedence over build.rustflags
-#[test]
+#[cargo_test]
 fn target_rustflags_precedence() {
     let p = project()
         .file("src/lib.rs", "")
@@ -1034,7 +1036,7 @@ fn target_rustflags_precedence() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn cfg_rustflags_normal_source() {
     let p = project()
         .file("src/lib.rs", "pub fn t() {}")
@@ -1113,7 +1115,7 @@ fn cfg_rustflags_normal_source() {
 }
 
 // target.'cfg(...)'.rustflags takes precedence over build.rustflags
-#[test]
+#[cargo_test]
 fn cfg_rustflags_precedence() {
     let p = project()
         .file("src/lib.rs", "pub fn t() {}")
@@ -1194,7 +1196,7 @@ fn cfg_rustflags_precedence() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn target_rustflags_string_and_array_form1() {
     let p1 = project()
         .file("src/lib.rs", "")
@@ -1239,7 +1241,7 @@ fn target_rustflags_string_and_array_form1() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn target_rustflags_string_and_array_form2() {
     let p1 = project()
         .file(
@@ -1290,7 +1292,7 @@ fn target_rustflags_string_and_array_form2() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn two_matching_in_config() {
     let p1 = project()
         .file(
@@ -1326,7 +1328,7 @@ fn two_matching_in_config() {
     p1.cargo("build").with_stderr("[FINISHED] [..]").run();
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_misspelled() {
     let p = project().file("src/main.rs", "fn main() { }").build();
 
@@ -1338,7 +1340,7 @@ fn env_rustflags_misspelled() {
     }
 }
 
-#[test]
+#[cargo_test]
 fn env_rustflags_misspelled_build_script() {
     let p = project()
         .file(
@@ -1359,3 +1361,75 @@ fn env_rustflags_misspelled_build_script() {
         .with_stderr_contains("[WARNING] Cargo does not read `RUST_FLAGS` environment variable. Did you mean `RUSTFLAGS`?")
         .run();
 }
+
+#[cargo_test]
+fn remap_path_prefix_ignored() {
+    // Ensure that --remap-path-prefix does not affect metadata hash.
+    let p = project().file("src/lib.rs", "").build();
+    p.cargo("build").run();
+    let rlibs = p
+        .glob("target/debug/deps/*.rlib")
+        .collect::<Result<Vec<_>, _>>()
+        .unwrap();
+    assert_eq!(rlibs.len(), 1);
+    p.cargo("clean").run();
+
+    let check_metadata_same = || {
+        let rlibs2 = p
+            .glob("target/debug/deps/*.rlib")
+            .collect::<Result<Vec<_>, _>>()
+            .unwrap();
+        assert_eq!(rlibs, rlibs2);
+    };
+
+    p.cargo("build")
+        .env(
+            "RUSTFLAGS",
+            "--remap-path-prefix=/abc=/zoo --remap-path-prefix /spaced=/zoo",
+        )
+        .run();
+    check_metadata_same();
+
+    p.cargo("clean").run();
+    p.cargo("rustc -- --remap-path-prefix=/abc=/zoo --remap-path-prefix /spaced=/zoo")
+        .run();
+    check_metadata_same();
+}
+
+#[cargo_test]
+fn remap_path_prefix_works() {
+    // Check that remap-path-prefix works.
+    Package::new("bar", "0.1.0")
+        .file("src/lib.rs", "pub fn f() -> &'static str { file!() }")
+        .publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+
+            [dependencies]
+            bar = "0.1"
+            "#,
+        )
+        .file(
+            "src/main.rs",
+            r#"
+            fn main() {
+                println!("{}", bar::f());
+            }
+            "#,
+        )
+        .build();
+
+    p.cargo("run")
+        .env(
+            "RUSTFLAGS",
+            format!("--remap-path-prefix={}=/foo", paths::root().display()),
+        )
+        .with_stdout("/foo/home/.cargo/registry/src/[..]/bar-0.1.0/src/lib.rs")
+        .run();
+}
